options(echo=FALSE)

#This file is used to create Table 4 of the manuscript (the power function of the data driven Neyman's test for the exponential example)
#If run as is (simply execute the code without changing anything) the output (it takes about 30 hours on a intel core i7 870 PC, using windows 7 and R version 2.14) will be identical to Table 4
#the random seed is controled by the set.seed() command in row  80 below
#For experimenting with the code, the inputs are:

# 1) Specify the H_0 and H_1 distributions: The options supported so far are "expon" for exponential, "weib" for weibull, "gamma" for gamma - this is done on line 86
# 2) Specify the distribution parameters in line 82 (current version uses values so that the distance between H_0 and H_1 is increasignly deviant with respect to Kullback-Liebler ), the user is free to set their own parameters.


library(surv2sample)



PowerFunction<-function(ItNumber, SampleSizeVector, CensParamVector, CritValsVector, H0dist, H1dist,  p1,p2, p3,p4)
{
  m<-length(SampleSizeVector) #calculate rejection percenatges for all sample sizes
  cUse1<-length(p3) #the output matrix will have as amny columns as the number of distribution parameters specified.
  PowerMatOut<-matrix(nrow=m, ncol=1+cUse1) #initialize output matrix
 
  for(i in 1:m)
  {
    SampleSizeUse<-SampleSizeVector[i] #choose the sample size from the sample size vector
    CensParamuse<-CensParamVector[i] #choose the censoring percentage of the sample
    CritValsUse<-CritValsVector[i] #choose the critical value (its actually the same for all calculations because it is given in terms of p-values)
    SampleIndices<- 1:SampleSizeUse
    TestValues<-sapply(1:ItNumber, function(i, SampleSizeUse, CensParamuse, H0dist, H1dist,  p1,p2, p3,p4,cUse1, SampleIndices)
    {
      cUse<- CensParamuse #100* length(which(data<cens))/SampleSizeUse
      AllTestValues<-sapply(1:cUse1, function(j,   SampleIndices, H0dist, H1dist, p1,p2, p3,p4)
      {  
       p3use<-p3[j]
      
       dataH0<-switch(H0dist, weib=rweibull(SampleSizeUse, p1,p2), gamma=rgamma(SampleSizeUse,p1,p2), expon=rexp(SampleSizeUse)) #generate data under H_0
       dataH1<-switch(H1dist, weib=rweibull(SampleSizeUse, p3use,p3use), gamma=rgamma(SampleSizeUse,p3use,p3use), expon=rexp(SampleSizeUse, rate=p3use))#generate data under H_1
       CensInd<-sample(SampleIndices, CensParamuse*SampleSizeUse/100) #select the indices of the sample which are considered censored
      
      
       cen <- rep(1, times=SampleSizeUse) #initialize censoring indicators
       cen[CensInd]<-0 # censored observations are marked with 0
      
       data<-c(dataH0, dataH1) #combine data for formulating the Surv object (required for Neyman's test routine in the surv2sample package)
       cen.all<-c(cen, rep(1, SampleSizeUse)) #combine censoring indicators for formulating the Surv object (required for Neyman's test routine in the surv2sample package)
       group.use<-c(rep(1, SampleSizeUse), rep(2,SampleSizeUse)) #formulate group indicators (required for Neyman's test routine in the surv2sample package)
       a<-Surv(data, cen.all) #formulate survival object
      
       test.use<-surv2.neyman(a,group.use, data.driven=TRUE, nsim=1000, approx="boot") #call Neyman'stest routine
       test.use$pval #extract the p-value
      },  SampleIndices, H0dist, H1dist, p1,p2, p3,p4 )
      
      
      c(cUse,   AllTestValues) #TestStatistic(data, cen, data,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4)  	)					
    }, SampleSizeUse, CensParamuse, H0dist, H1dist,  p1,p2,p3,p4, cUse1, SampleIndices)
    
    
    TestValues<-matrix(TestValues, ncol=1+cUse1, byrow=T) #put all results in matrix format
    PowerMatOut[i,1]<- TestValues[1]
    TestValues<-TestValues[,c(2:ncol(TestValues))]
    dout<-vector(length=ncol(TestValues))
    for(k in 1:ncol(TestValues))
    { 
      TestValUse<- TestValues[,k]
      
      dout[k] <- 100*length(TestValUse[which(TestValUse<CritValsUse)])/ItNumber #calculate rejection percentages
    }
    PowerMatOut[i,c(2:ncol(PowerMatOut))]<- dout #store the result in the appropriate matrix place
  }
  PowerMatOut
}


SampleSizeVector<-rep(c(30,	40,	50,	60,	70,	80,	90,	100),4)#define the sample sizes (30-100), replicate 4 times, one for each censoring percentage
CensParamVector <- rep((0:3)*10,each=8)#define the censoring percentages of the samples, so that they match the sample sizes and all unique combinations are created
critic.vals<- c(  0.05,0.05,0.05,0.05,0.05,0.05,0.05,0.05 )#instead of cut-off points here we need the p-values which put a threshhold on the acceptance confidence level

CritValsVector<- rep(critic.vals, 4)#replicate the critical values so that they match the sample size/censoring combinations

set.seed(157)  #use this to get exactly the same numbers as in the paper
ItNumber<-10000 #number of iterations over which the rejection percentages are calculated

DistParameters<-c(1,1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2, 2.1, 2.2) #specify the null and 12 H_1 variants distribution parameters

ResultsMat<-matrix(nrow = 4*8, ncol=2*length(DistParameters)) #initialize output matrix

arg<-PowerFunction(ItNumber, SampleSizeVector, CensParamVector, CritValsVector, "expon", "expon",   1,1, DistParameters,1) 
ResultsMat<-arg #store the rejection percenatages in the matrix created previously
SScol<-SampleSizeVector#rep(SampleSizeVector[c(1:8)], times=5)
MyDistName<-paste("exp(", DistParameters, ")") #assign names to the matrix columns
ResultsMat<- cbind( SScol, ResultsMat) # bind the sample sizes to the power matrix (used for pretty printing)
a<-c( "samplesize", "Cens", MyDistName[1:length(DistParameters)]) #formulate the names of the columns of the power matrix (used for pretty printing)

colnames(ResultsMat)<-a #assign the column names to the power matrix (used for pretty printing)

ResultsMat  #return the final result
